/*$Id: path.h,v 1.43 2001/06/15 21:53:20 al Exp $ -*- C++ -*-
 */
#ifndef IBIS_PATH_H_INCLUDED
#define IBIS_PATH_H_INCLUDED
#include "base.h"
/*--------------------------------------------------------------------------*/
class Path_Item_Base
  :public Base
{
protected:
  std::string _name;
public:
  explicit Path_Item_Base() {}
  void parse(CS&) = 0;
  void dump(std::ostream& o)const = 0;
  const std::string& name()const {return _name;}
};
/*--------------------------------------------------------------------------*/
class Path
  :public List_Base<Path_Item_Base>
{
  /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
public:
  class Pin
    :public Path_Item_Base
  {
  private:
    void parse(CS&);
    void dump(std::ostream& o)const;
  public:
    explicit Pin(CS& file) {parse(file);}
  };
  /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  class Seg
    :public Path_Item_Base
  {
  private:
    Float _len;
    Float _r;
    Float _l;
    Float _c;
    void parse(CS&);
    void dump(std::ostream& o)const;
  public:
    explicit Seg(CS& file): _len(0.), _r(0.), _l(0.), _c(0.) {parse(file);}
    bool   lumped()const{return _len == 0;}
    double len()const	{return _len;}
    double r()const	{return _r;}
    double l()const	{return _l;}
    double c()const	{return _c;}
    double z0()const	{return sqrt(_l / _c);}
    double delay()const	{return _len * sqrt(_l * _c);}
  };
  /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  class Fork
    :public Path_Item_Base, public List_Base<Path_Item_Base>
  {
  private:
    void parse(CS&);
    void dump(std::ostream& o)const;
  public:
    explicit Fork(CS& file) {parse(file);}
  };
  /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  class Node
    :public Path_Item_Base
  {
  private:
    void parse(CS&);
    void dump(std::ostream& o)const;
  public:
    explicit Node(CS& file) {parse(file);}
  };
  /*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
private:
  std::string _name;
  void parse(CS&);
  void dump(std::ostream& o)const;
public:
  explicit Path(CS& file) {parse(file);}
  const std::string&	name()const  {return _name;}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
